/***
 * REXX Communication Control Function Library  Ver.1.9
 *                  copyright 1998-2002, Naga(Kazuo Adachi)
 *                          mailto:nagaw@attglobal.net
 *
 * $Id$
 *
 * $Log$
 */

#include "RxComm.h"
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/*--------------------------------------------------------*/
/***
 * Parity(Line Control) Mode REXX String
 */
static PCSZ szParity = "NnOoEeMmSs";

/*--------------------------------------------------------*/
/**
 * Communication Port Setup Sub-Routine.
 */
static APIRET setupCommPort( ULONG argc, RXSTRING argv[] )
{
    APIRET      retCode;
    HFILE       hcomFile;
    EXSETBAUD   exBaud;
    USHORT      unBaud;
    UINT        uParam;
    BYTE        bParam;
    PSZ         pszChar;
    LINECONTROL lcParam;
    DCBINFO     dcbInfo;

    hcomFile = rxcInfo.hcmFile;

    /**
     * build COM device parameters.
     */
    if ( (0 < argc) && (0 < RXSTRLEN(argv[0])) )
    {
        exBaud.ulBaud = atol( RXSTRPTR(argv[0]) );
        exBaud.ucBaud = 0;
        retCode = DosDevIOCtl( hcomFile, IOCTL_ASYNC, ASYNC_EXTSETBAUDRATE,
            &exBaud, sizeof(EXSETBAUD), NULL, NULL, 0L, NULL );
        if ( NO_ERROR != retCode )
        {
            unBaud = exBaud.ulBaud;
            retCode = DosDevIOCtl( hcomFile, IOCTL_ASYNC, ASYNC_SETBAUDRATE,
                &unBaud, sizeof(USHORT), NULL, NULL, 0L, NULL );
        }
        if ( NO_ERROR != retCode )
        {
            return retCode;
        }
    }

    if ( (1 < argc) && (0 < RXSTRLEN(argv[1])) )
    {
        uParam = 0;
        bParam = (BYTE)(RXSTRPTR(argv[1])[uParam] - '0');
        if ( (5 > bParam) || (8 < bParam) )
        {
            lcParam.bDataBits   = 7;
        }
        else
        {
            lcParam.bDataBits   = bParam;
            uParam++;
        }
        pszChar = strchr( szParity, RXSTRPTR(argv[1])[uParam] );
        if ( NULL == pszChar )
        {
            lcParam.bParity = 2;
        }
        else
        {
            lcParam.bParity = (pszChar - szParity) / 2;
            uParam++;
        }
        lcParam.bStopBits   = 0;
        if ( ('1' == RXSTRPTR(argv[1])[uParam  ])
          && ('.' == RXSTRPTR(argv[1])[uParam+1])
          && ('5' == RXSTRPTR(argv[1])[uParam+2]) )
        {
            lcParam.bStopBits = 1;
        }
        else
        if ( '2' == RXSTRPTR(argv[1])[uParam] )
        {
            lcParam.bStopBits = 2;
        }
        lcParam.fTransBreak = 0;
        retCode = DosDevIOCtl( hcomFile, IOCTL_ASYNC, ASYNC_SETLINECTRL,
            &lcParam, sizeof(LINECONTROL), NULL, NULL, 0L, NULL );
        if ( NO_ERROR != retCode )
        {
            return retCode;
        }
    }

    retCode = DosDevIOCtl( hcomFile, IOCTL_ASYNC, ASYNC_GETDCBINFO,
        NULL, 0L, NULL, &dcbInfo, sizeof(DCBINFO), NULL );
    if ( NO_ERROR != retCode )
    {
        return retCode;
    }
    dcbInfo.usWriteTimeout = DCB_WRITETIME;
    dcbInfo.usReadTimeout  = DCB_READ_TIME;
    dcbInfo.fbTimeout      = DCB_TIME_OUT;
    dcbInfo.bErrorReplacementChar = 0x00;
    dcbInfo.bBreakReplacementChar = 0x00;
    if ( (2 < argc) && (0 < RXSTRLEN(argv[2])) )
    {
        dcbInfo.fbCtlHndShake  = DCB_MODESHAKE;
        dcbInfo.fbFlowReplace  = DCB_MODE_FLOW;
        if ( NULL != strstr( RXSTRPTR(argv[2]), "XON/XOFF" ) )
        {
            dcbInfo.fbFlowReplace |= MODE_AUTO_TRANSMIT | MODE_AUTO_RECEIVE;
            dcbInfo.bXONChar       = 0x11;
            dcbInfo.bXOFFChar      = 0x13;
        }
        else
        {
            dcbInfo.bXONChar       = 0x00;
            dcbInfo.bXOFFChar      = 0x00;
        }
        if ( (NULL != strstr( RXSTRPTR(argv[2]), "RTS/CTS" ))
          || (NULL != strstr( RXSTRPTR(argv[2]),  "RS/CS"  )) )
        {
            dcbInfo.fbCtlHndShake |= MODE_CTS_HANDSHAKE;
            dcbInfo.fbFlowReplace |= MODE_RTS_HANDSHAKE;
        }
        else
        {
            dcbInfo.fbFlowReplace |= MODE_RTS_CONTROL;
        }
        if ( (NULL != strstr( RXSTRPTR(argv[2]), "DTR/DSR" ))
          || (NULL != strstr( RXSTRPTR(argv[2]),  "ER/DR"  )) )
        {
            dcbInfo.fbCtlHndShake |= MODE_DTR_HANDSHAKE;
            dcbInfo.fbFlowReplace |= MODE_DSR_HANDSHAKE;
        }
        else
        {
            dcbInfo.fbCtlHndShake |= MODE_DTR_CONTROL;
        }
    }
    if ( rxcInfo.nulStrp )
    {
        dcbInfo.fbFlowReplace |= MODE_NULL_STRIPPING;
    }
    retCode = DosDevIOCtl( hcomFile, IOCTL_ASYNC, ASYNC_SETDCBINFO,
        &dcbInfo, sizeof(DCBINFO), NULL, NULL, 0L, NULL );
    if ( NO_ERROR != retCode )
    {
        return retCode;
    }

    return NO_ERROR;
}

/*--------------------------------------------------------*/
/*--------------------------------------------------------*/
/**
 * REXX Communication Setup Function.
 */
ULONG APIENTRY RxCommSetup(PSZ name,ULONG argc,RXSTRING argv[],PSZ queue,PRXSTRING retstr)
{
    APIRET retCode;

    retstr->strlength = 0;
    if ( 3 < argc )
    {
        return 40;
    }
    if ( NULLHANDLE == rxcInfo.hcmFile )
    {
        return 40;
    }

    retCode = setupCommPort( argc, argv );

    sprintf( retstr->strptr, "%d", retCode );
    retstr->strlength = strlen( retstr->strptr );

    return 0;
}

/*--------------------------------------------------------*/
/*--------------------------------------------------------*/
/**
 * REXX Communication Open Function.
 */
ULONG APIENTRY RxCommOpen(PSZ name,ULONG argc,RXSTRING argv[],PSZ queue,PRXSTRING retstr)
{
    APIRET      retCode;
    PSZ         pszName;
    HFILE       hcomFile;
    ULONG       ulAction;
    MODEMSTATUS msParam;
    USHORT      uComErr;
    UCHAR       ucFlush;

    retstr->strlength = 0;
    if ( (1 > argc) || (4 < argc) )
    {
        return 40;
    }
    if ( (0 != strnicmp( RXSTRPTR(argv[0]), "COM", 3 ))
      || (4 != RXSTRLEN(argv[0])) )
    {
        return 40;
    }
    if ( NULLHANDLE != rxcInfo.hcmFile )
    {
        sprintf( retstr->strptr, "%d", ERROR_OPEN_FAILED );
        retstr->strlength = strlen( retstr->strptr );
        return 0;
    }

    /**
     * create COM port device handle.
     */
    pszName = RXSTRPTR(argv[0]);
    retCode = DosOpen( pszName, &hcomFile, &ulAction, 0L, FILE_NORMAL,
        FILE_OPEN, OPEN_ACCESS_READWRITE | OPEN_SHARE_DENYNONE, 0L );

    sprintf( retstr->strptr, "%d", retCode );
    retstr->strlength = strlen( retstr->strptr );
    if ( NO_ERROR != retCode )
    {
        return 0;
    }

    /**
     * makeup rxcInfo parameter.
     */
    rxcInfo.pszName = pszName;
    rxcInfo.hcmFile = hcomFile;

    rxcInfo.pszRecv = malloc( rxcInfo.maxRecv );
    rxcInfo.posRecv = 0;

    rxcInfo.pszSend = malloc( rxcInfo.maxSend );

    /**
     * build COM device parameters.
     */
    retCode = setupCommPort( argc-1, &argv[1] );

    /**
     * wakeup DTR/RTS signals.
     */
    msParam.fbModemOn  = 0x03;
    msParam.fbModemOff = 0xff;
    retCode = DosDevIOCtl( hcomFile, IOCTL_ASYNC, ASYNC_SETMODEMCTRL,
        &msParam, sizeof(MODEMSTATUS), NULL,
        &uComErr, sizeof(USHORT)     , NULL );

    /**
     * flush COM device queue.
     */
    DosSleep ( 10 );
    ucFlush = 0;
    retCode = DosDevIOCtl( hcomFile, IOCTL_GENERAL, DEV_FLUSHOUTPUT,
        &ucFlush, sizeof(UCHAR), NULL, NULL, 0L, NULL );
    DosSleep ( 10 );
    ucFlush = 0;
    retCode = DosDevIOCtl( hcomFile, IOCTL_GENERAL, DEV_FLUSHINPUT,
        &ucFlush, sizeof(UCHAR), NULL, NULL, 0L, NULL );

    return 0;
}

/**
 * REXX Communication Close Function.
 */
ULONG APIENTRY RxCommClose(PSZ name,ULONG argc,RXSTRING argv[],PSZ queue,PRXSTRING retstr)
{
    APIRET      retCode;
    HFILE       hcomFile;
    MODEMSTATUS msParam;
    USHORT      uComErr;
    UCHAR       ucFlush;

    retstr->strlength = 0;
    if ( 0 < argc )
    {
        return 40;
    }
    if ( NULLHANDLE == rxcInfo.hcmFile )
    {
        return 40;
    }

    hcomFile = rxcInfo.hcmFile;

    DosEnterCritSec();

    /**
     * flush COM device queue.
     */
    DosSleep ( 10 );
    ucFlush = 0;
    retCode = DosDevIOCtl( hcomFile, IOCTL_GENERAL, DEV_FLUSHOUTPUT,
        &ucFlush, sizeof(UCHAR), NULL, NULL, 0L, NULL );
    DosSleep ( 10 );
    ucFlush = 0;
    retCode = DosDevIOCtl( hcomFile, IOCTL_GENERAL, DEV_FLUSHINPUT,
        &ucFlush, sizeof(UCHAR), NULL, NULL, 0L, NULL );

    /**
     * drop DTR/RTS signals.
     */
    msParam.fbModemOn  = 0x00;
    msParam.fbModemOff = 0xfc;
    retCode = DosDevIOCtl( hcomFile, IOCTL_ASYNC, ASYNC_SETMODEMCTRL,
        &msParam, sizeof(MODEMSTATUS), NULL,
        &uComErr, sizeof(USHORT)     , NULL );

    /**
     * close COM device.
     */
    retCode = DosClose( hcomFile );

    DosExitCritSec();

    if ( NO_ERROR == retCode )
    {
        if ( NULLHANDLE != rxcInfo.logFile )
        {
            DosClose( rxcInfo.logFile );
            rxcInfo.logFile = NULLHANDLE;
        }

        free( rxcInfo.pszSend );
        rxcInfo.pszSend = NULL;

        free( rxcInfo.pszRecv );
        rxcInfo.pszRecv = NULL;
        rxcInfo.posRecv = 0;

        rxcInfo.pszName = NULL;
        rxcInfo.hcmFile = NULLHANDLE;
    }

    sprintf( retstr->strptr, "%d", retCode );
    retstr->strlength = strlen( retstr->strptr );

    return 0;
}

