/***
 * REXX Communication Control Function Library  Ver.1.9
 *                  copyright 1998-2002, Naga(Kazuo Adachi)
 *                          mailto:nagaw@attglobal.net
 *
 * $Id$
 *
 * $Log$
 */

#include "RxComm.h"
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/*--------------------------------------------------------*/
/*--------------------------------------------------------*/
/**
 * REXX Communication Send Break Function.
 */
ULONG APIENTRY RxCommBreak(PSZ name,ULONG argc,RXSTRING argv[],PSZ queue,PRXSTRING retstr)
{
    APIRET retCode;
    APIRET subCode;
    HFILE  hcomFile;
    ULONG  ulSleep;
    USHORT uComErr;

    retstr->strlength = 0;
    if ( 1 < argc )
    {
        return 40;
    }
    if ( NULLHANDLE == rxcInfo.hcmFile )
    {
        return 40;
    }

    hcomFile = rxcInfo.hcmFile;

    /**
     * get break sending time.
     */
    ulSleep = DEF_SNDBREAK;
    if ( (0 < argc) && (0 < RXSTRLEN(argv[0])) )
    {
        ulSleep = atol( RXSTRPTR(argv[0]) );
    }

    /**
     * break signal 'ON'.
     */
    DosEnterCritSec();
    retCode = DosDevIOCtl( hcomFile, IOCTL_ASYNC, ASYNC_SETBREAKON,
        NULL, 0L, NULL, &uComErr, sizeof(USHORT), NULL );
    DosExitCritSec();

    if ( NO_ERROR == retCode )
    {
        DosSleep( ulSleep );
    }

    /**
     * break signal 'OFF'.
     */
    DosEnterCritSec();
    subCode = DosDevIOCtl( hcomFile, IOCTL_ASYNC, ASYNC_SETBREAKOFF,
        NULL, 0L, NULL, &uComErr, sizeof(USHORT), NULL );
    DosExitCritSec();

    if ( (NO_ERROR == retCode)
      && (NO_ERROR != subCode) )
    {
        retCode = subCode;
    }

    sprintf( retstr->strptr, "%d", retCode );
    retstr->strlength = strlen( retstr->strptr );

    return 0;
}

/*--------------------------------------------------------*/
/**
 * REXX Communication Control Signal Function.
 */
ULONG APIENTRY RxCommSignal(PSZ name,ULONG argc,RXSTRING argv[],PSZ queue,PRXSTRING retstr)
{
    APIRET      retCode;
    APIRET      subCode;
    HFILE       hcomFile;
    MODEMSTATUS msParam;
    USHORT      uComErr;
    UCHAR       uInpSgnl;
    UCHAR       uOutSgnl;

    retstr->strlength = 0;
    if ( 2 < argc )
    {
        return 40;
    }
    if ( NULLHANDLE == rxcInfo.hcmFile )
    {
        return 40;
    }

    hcomFile = rxcInfo.hcmFile;

    /**
     * control DTR/RTS signals.
     */
    msParam.fbModemOn  = 0x00;
    msParam.fbModemOff = 0xff;
    if ( (0 < argc) && (0 < RXSTRLEN(argv[0])) )
    {
        if ( 0 == stricmp( RXSTRPTR(argv[0]), "ON" ) )
        {
            msParam.fbModemOn  |= DTR_ON;
        }
        else
        if ( 0 == stricmp( RXSTRPTR(argv[0]), "OFF" ) )
        {
            msParam.fbModemOff &= DTR_OFF;
        }
    }
    if ( (1 < argc) && (0 < RXSTRLEN(argv[1])) )
    {
        if ( 0 == stricmp( RXSTRPTR(argv[1]), "ON" ) )
        {
            msParam.fbModemOn  |= RTS_ON;
        }
        else
        if ( 0 == stricmp( RXSTRPTR(argv[1]), "OFF" ) )
        {
            msParam.fbModemOff &= RTS_OFF;
        }
    }

    DosEnterCritSec();

    retCode = DosDevIOCtl( hcomFile, IOCTL_ASYNC, ASYNC_SETMODEMCTRL,
        &msParam, sizeof(MODEMSTATUS), NULL,
        &uComErr, sizeof(USHORT)     , NULL );

    /**
     * query modem signals.
     */
    subCode = DosDevIOCtl( hcomFile, IOCTL_ASYNC, ASYNC_GETMODEMOUTPUT,
        NULL, 0L, NULL, &uOutSgnl, sizeof(UCHAR), NULL );
    if ( (NO_ERROR == retCode)
      && (NO_ERROR != subCode) )
    {
        retCode = subCode;
    }

    subCode = DosDevIOCtl( hcomFile, IOCTL_ASYNC, ASYNC_GETMODEMINPUT,
        NULL, 0L, NULL, &uInpSgnl, sizeof(UCHAR), NULL );
    if ( (NO_ERROR == retCode)
      && (NO_ERROR != subCode) )
    {
        retCode = subCode;
    }

    DosExitCritSec();

    /**
     * build result code string.
     */
    if ( NO_ERROR != retCode )
    {
        sprintf( retstr->strptr, "%d", retCode );
        retstr->strlength = strlen( retstr->strptr );
        return 0;
    }

    strcpy( retstr->strptr, "" );

    if ( DTR_ON & uOutSgnl )
    {
        if ( 0 < strlen( retstr->strptr ) ){ strcat( retstr->strptr, " " ); }
        strcat( retstr->strptr, "DTR" );
    }
    if ( RTS_ON & uOutSgnl )
    {
        if ( 0 < strlen( retstr->strptr ) ){ strcat( retstr->strptr, " " ); }
        strcat( retstr->strptr, "RTS" );
    }
    if ( CTS_ON & uInpSgnl )
    {
        if ( 0 < strlen( retstr->strptr ) ){ strcat( retstr->strptr, " " ); }
        strcat( retstr->strptr, "CTS" );
    }
    if ( DSR_ON & uInpSgnl )
    {
        if ( 0 < strlen( retstr->strptr ) ){ strcat( retstr->strptr, " " ); }
        strcat( retstr->strptr, "DSR" );
    }
    if ( RI_ON  & uInpSgnl )
    {
        if ( 0 < strlen( retstr->strptr ) ){ strcat( retstr->strptr, " " ); }
        strcat( retstr->strptr, "RI" );
    }
    if ( DCD_ON & uInpSgnl )
    {
        if ( 0 < strlen( retstr->strptr ) ){ strcat( retstr->strptr, " " ); }
        strcat( retstr->strptr, "DCD" );
    }

    if ( RX_QUE_OVERRUN      & uComErr )
    {
        if ( 0 < strlen( retstr->strptr ) ){ strcat( retstr->strptr, " " ); }
        strcat( retstr->strptr, "QOVRRUN" );
    }
    if ( RX_HARDWARE_OVERRUN & uComErr )
    {
        if ( 0 < strlen( retstr->strptr ) ){ strcat( retstr->strptr, " " ); }
        strcat( retstr->strptr, "HOVRRUN" );
    }
    if ( PARITY_ERROR        & uComErr )
    {
        if ( 0 < strlen( retstr->strptr ) ){ strcat( retstr->strptr, " " ); }
        strcat( retstr->strptr, "EPARITY" );
    }
    if ( FRAMING_ERROR       & uComErr )
    {
        if ( 0 < strlen( retstr->strptr ) ){ strcat( retstr->strptr, " " ); }
        strcat( retstr->strptr, "EFRAME" );
    }

    retstr->strlength = strlen( retstr->strptr );

    return 0;
}


/*--------------------------------------------------------*/
/*--------------------------------------------------------*/
/**
 * REXX Communication Option Function.
 */
static PSZ  szRXONOFF[] = {"OFF", "ON"};
static BOOL RxCommONOFF( PSZ pszValue, BOOL isDef )
{
    BOOL rxONOFF;

    for ( rxONOFF = FALSE; rxONOFF <= TRUE; rxONOFF++ )
    {
        if ( 0 == stricmp( pszValue, szRXONOFF[rxONOFF] ) )
        {
            return rxONOFF;
        }
    }
    return isDef;
}
static PSZ  szRXCRLF[] = {"NULL", "CR", "LF", "CRLF", "ORG"};
static RXCRLF RxCommCRLF( PSZ pszValue, RXCRLF rxDef )
{
    RXCRLF rxCRLF;

    for ( rxCRLF = RXCH_NULL; rxCRLF <= RXCH_ORG; rxCRLF++ )
    {
        if ( 0 == stricmp( pszValue, szRXCRLF[rxCRLF] ) )
        {
            return rxCRLF;
        }
    }
    return rxDef;
}
ULONG APIENTRY RxCommOption(PSZ name,ULONG argc,RXSTRING argv[],PSZ queue,PRXSTRING retstr)
{
    APIRET  retCode;
    HFILE   hcomFile;
    DCBINFO dcbInfo;

    retstr->strlength = 0;
    if ( 2 != argc )
    {
        return 40;
    }

    hcomFile = rxcInfo.hcmFile;

    strcpy( retstr->strptr, "" );

    if ( 0 == stricmp( RXSTRPTR(argv[0]), "NULL_STRIP" ) )
    {
        strcpy( retstr->strptr, szRXONOFF[rxcInfo.nulStrp] );
        rxcInfo.nulStrp = RxCommONOFF( RXSTRPTR(argv[1]), rxcInfo.nulStrp );

        if ( NULLHANDLE != hcomFile )
        {
            retCode = DosDevIOCtl( hcomFile, IOCTL_ASYNC, ASYNC_GETDCBINFO,
                NULL, 0L, NULL, &dcbInfo, sizeof(DCBINFO), NULL );
            if ( rxcInfo.nulStrp )
            {
                dcbInfo.fbFlowReplace |=  MODE_NULL_STRIPPING;
            }
            else
            {
                dcbInfo.fbFlowReplace &= ~MODE_NULL_STRIPPING;
            }
            retCode = DosDevIOCtl( hcomFile, IOCTL_ASYNC, ASYNC_SETDCBINFO,
                &dcbInfo, sizeof(DCBINFO), NULL, NULL, 0L, NULL );
        }
    }
    else
    if ( 0 == stricmp( RXSTRPTR(argv[0]), "SEND_CTLCH" ) )
    {
        strcpy( retstr->strptr, szRXONOFF[rxcInfo.sndChar] );
        rxcInfo.sndChar = RxCommONOFF( RXSTRPTR(argv[1]), rxcInfo.sndChar );
    }
    else
    if ( 0 == stricmp( RXSTRPTR(argv[0]), "RECV_CTLCH" ) )
    {
        strcpy( retstr->strptr, szRXONOFF[rxcInfo.rcvChar] );
        rxcInfo.rcvChar = RxCommONOFF( RXSTRPTR(argv[1]), rxcInfo.rcvChar );
    }
    else
    if ( 0 == stricmp( RXSTRPTR(argv[0]), "MODE_LOGOP" ) )
    {
        if ( rxcInfo.logMode )
        {
            strcpy( retstr->strptr, "CREATE" );
        }
        else
        {
            strcpy( retstr->strptr, "APPEND" );
        }
        if ( 0 == strnicmp( RXSTRPTR(argv[1]), "CREATE", RXSTRLEN(argv[1]) ) )
        {
            rxcInfo.logMode = TRUE;
        }
        else
        if ( 0 == strnicmp( RXSTRPTR(argv[1]), "APPEND", RXSTRLEN(argv[1]) ) )
        {
            rxcInfo.logMode = FALSE;
        }
    }
    else
    if ( 0 == stricmp( RXSTRPTR(argv[0]), "MODE_BINCH" ) )
    {
        strcpy( retstr->strptr, szRXONOFF[rxcInfo.binMode] );
        rxcInfo.binMode = RxCommONOFF( RXSTRPTR(argv[1]), rxcInfo.binMode );
    }
    else
    if ( 0 == stricmp( RXSTRPTR(argv[0]), "SEND_CNVCR" ) )
    {
        strcpy( retstr->strptr, szRXCRLF[rxcInfo.sndRxCR] );
        rxcInfo.sndRxCR = RxCommCRLF( RXSTRPTR(argv[1]), rxcInfo.sndRxCR );
    }
    else
    if ( 0 == stricmp( RXSTRPTR(argv[0]), "SEND_CNVLF" ) )
    {
        strcpy( retstr->strptr, szRXCRLF[rxcInfo.sndRxLF] );
        rxcInfo.sndRxLF = RxCommCRLF( RXSTRPTR(argv[1]), rxcInfo.sndRxLF );
    }
    else
    if ( 0 == stricmp( RXSTRPTR(argv[0]), "RECV_CNVCR" ) )
    {
        strcpy( retstr->strptr, szRXCRLF[rxcInfo.rcvRxCR] );
        rxcInfo.rcvRxCR = RxCommCRLF( RXSTRPTR(argv[1]), rxcInfo.rcvRxCR );
    }
    else
    if ( 0 == stricmp( RXSTRPTR(argv[0]), "RECV_CNVLF" ) )
    {
        strcpy( retstr->strptr, szRXCRLF[rxcInfo.rcvRxLF] );
        rxcInfo.rcvRxLF = RxCommCRLF( RXSTRPTR(argv[1]), rxcInfo.rcvRxLF );
    }

    retstr->strlength = strlen( retstr->strptr );

    return 0;
}

/*--------------------------------------------------------*/
/*--------------------------------------------------------*/
/**
 * REXX Communication Log Function.
 */
ULONG APIENTRY RxCommLog(PSZ name,ULONG argc,RXSTRING argv[],PSZ queue,PRXSTRING retstr)
{
    APIRET retCode;
    BOOL   logCreate;
    HFILE  hLogFile;
    ULONG  ulAction;
    ULONG  ulOpFlag;
    ULONG  ulOpMode;
    ULONG  ulNewPtr;

    retstr->strlength = 0;
    if ( 2 < argc )
    {
        return 40;
    }
    logCreate = rxcInfo.logMode;
    if ( (1 < argc) && (0 < RXSTRLEN(argv[1])) )
    {
        if ( 0 == strnicmp( RXSTRPTR(argv[1]), "CREATE", RXSTRLEN(argv[1]) ) )
        {
            logCreate = TRUE;
        }
        else
        if ( 0 == strnicmp( RXSTRPTR(argv[1]), "APPEND", RXSTRLEN(argv[1]) ) )
        {
            logCreate = FALSE;
        }
    }

    if ( NULLHANDLE != rxcInfo.logFile )
    {
        retCode = DosClose( rxcInfo.logFile );
        rxcInfo.logFile = NULLHANDLE;
    }

    if ( (0 < argc) && (0 < RXSTRLEN(argv[0])) )
    {
        hLogFile = 0L;
        ulAction = 0L;
        ulOpFlag = OPEN_ACTION_CREATE_IF_NEW;
        if ( logCreate )
        {
            ulOpFlag |= OPEN_ACTION_REPLACE_IF_EXISTS;
        }
        else
        {
            ulOpFlag |= OPEN_ACTION_OPEN_IF_EXISTS;
        }
        ulOpMode = OPEN_ACCESS_WRITEONLY | OPEN_SHARE_DENYWRITE
                 | OPEN_FLAGS_SEQUENTIAL | OPEN_FLAGS_NO_CACHE;

        retCode = DosOpen( RXSTRPTR(argv[0]), &hLogFile, &ulAction, 0L, FILE_ARCHIVED | FILE_NORMAL,
            ulOpFlag, ulOpMode, 0L );
        if ( NO_ERROR == retCode )
        {
            DosSetFilePtr( hLogFile, 0L, FILE_END, &ulNewPtr );
            rxcInfo.logFile = hLogFile;
        }
    }

    sprintf( retstr->strptr, "%d", retCode );
    retstr->strlength = strlen( retstr->strptr );

    return 0;
}

