/***
 * REXX Communication Control Function Library  Ver.1.9
 *                  copyright 1998-2002, Naga(Kazuo Adachi)
 *                          mailto:nagaw@attglobal.net
 *
 * $Id$
 *
 * $Log$
 */

#include "RxComm.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/*--------------------------------------------------------*/
/*--------------------------------------------------------*/
/**
 * REXX Communication Get Time Function.
 */
static ULONG getTimerCount( VOID )
{
    ULONG ulTime;

    DosQuerySysInfo( QSV_MS_COUNT, QSV_MS_COUNT, &ulTime, sizeof(ULONG) );
    return ulTime;
}

/*--------------------------------------------------------*/
/**
 * check MS-KANJI(SJIS/DBCS) first byte.
 */
static BOOL isJms1( UINT chJms1 )
{
    chJms1 &= 0x00ff;

    if ( (0x0080 < chJms1) && (0x00a0 > chJms1) )
    {
        return TRUE;
    }
    if ( (0x00df < chJms1) && (0x00fd > chJms1) )
    {
        return TRUE;
    }
    return FALSE;
}
/**
 * check MS-KANJI(SJIS/DBCS) second byte.
 */
static BOOL isJms2( UINT chJms2 )
{
    chJms2 &= 0x00ff;

    if ( (0x003f < chJms2) && (0x00fd > chJms2) )
    {
        if ( 0x007f == chJms2 )
        {
            return FALSE;
        }
        return TRUE;
    }
    return FALSE;
}

/*--------------------------------------------------------*/
/*--------------------------------------------------------*/
/**
 * REXX Communication Read Function.
 */
ULONG APIENTRY RxCommRead(PSZ name,ULONG argc,RXSTRING argv[],PSZ queue,PRXSTRING retstr)
{
    APIRET  retCode;
    HFILE   hcomFile;
    RXQUEUE rxQueue;
    ULONG   ulSize;
    CHAR    chLast;
    RXCRLF  rxCRLF;
    BOOL    isDBCS;
    BOOL    isExit;
    BOOL    isWait;
    ULONG   ulWait;
    ULONG   ulTime;
    ULONG   ulBuff;

    retstr->strlength = 0;
    if ( 2 < argc )
    {
        return 40;
    }
    if ( NULLHANDLE == rxcInfo.hcmFile )
    {
        return 40;
    }

    hcomFile = rxcInfo.hcmFile;

    /**
     * check timeout option value.
     */
    if ( (0 == argc) || (0 == RXSTRLEN(argv[0])) )
    {
        isWait = TRUE;
        ulWait = 0;
    }
    else
    if ( -1 == atol( RXSTRPTR(argv[0]) ) )
    {
        isWait = TRUE;
        ulWait = 0;
    }
    else
    {
        isWait = FALSE;
        ulWait = atol( RXSTRPTR(argv[0]) ) * 10;
    }

    ulBuff = RXAUTOBUFLEN;
    if ( (1 < argc) && (0 < RXSTRLEN(argv[1])) )
    {
        ulBuff = atol( RXSTRPTR(argv[1]) );
    }

    /**
     * read(recv) string from COM device.
     */
    isExit = FALSE;
    ulTime = getTimerCount();
    ulSize = 0;
    isDBCS = FALSE;
    chLast = 0x00;
    while ( 0x0a != chLast )
    {
        if ( ulBuff <= retstr->strlength )
        {
            break;
        }

        /**
         * recieve chars from COM device.
         */
        if ( 0 == rxcInfo.posRecv )
        {
            DosEnterCritSec();
            retCode = DosRead( hcomFile, rxcInfo.pszRecv, rxcInfo.maxRecv, &ulSize );
            DosExitCritSec();
            if ( NO_ERROR != retCode )
            {
                break;
            }

            rxcInfo.posRecv = ulSize;
            if ( 0 < ulSize )
            {
                isExit = FALSE;
                ulTime = getTimerCount();
            }
            else
            if ( rxcInfo.binMode || !isDBCS )
            {
                /**
                 * check timeout
                 */
                if ( !isWait && (0 == ulWait) )
                {
                    break;
                }
                if ( isExit &&  isWait
                  && (0 < retstr->strlength) )
                {
                    break;
                }
                if ( isExit && !isWait
                  && (ulWait <= getTimerCount() - ulTime) )
                {
                    break;
                }

                DosSleep( DEF_READWAIT );
                isExit = TRUE;
            }
            else
            if ( isExit && (DEF_READEXIT <= getTimerCount() - ulTime) )
            {
                break;
            }

            ulSize = 0;
        }

        /**
         * copy recieved char to result buffer.
         */
        if ( 0 < rxcInfo.posRecv )
        {
            if ( !rxcInfo.binMode
              && (0x0d == chLast)
              && (0x0a != rxcInfo.pszRecv[ulSize]) )
            {
                break;
            }

            chLast = rxcInfo.pszRecv[ulSize++];
            rxCRLF = RXCH_ORG;
            if ( '\x0d' == chLast )
            {
                rxCRLF = rxcInfo.rcvRxCR;
            }
            else
            if ( '\x0a' == chLast )
            {
                rxCRLF = rxcInfo.rcvRxLF;
            }
            switch ( rxCRLF )
            {
                case RXCH_NULL:
                    break;
                case RXCH_CR:
                    retstr->strptr[retstr->strlength++] = 0x0d;
                    break;
                case RXCH_LF:
                    retstr->strptr[retstr->strlength++] = 0x0a;
                    break;
                case RXCH_CRLF:
                    retstr->strptr[retstr->strlength++] = 0x0d;
                    retstr->strptr[retstr->strlength++] = 0x0a;
                    break;
                case RXCH_ORG:
                    retstr->strptr[retstr->strlength++] = chLast;
                    break;
            }
            rxcInfo.posRecv--;

            if ( isDBCS )
            {
                isDBCS = FALSE;
            }
            else
            {
                isDBCS = isJms1( chLast );
            }
        }
    }
    retstr->strptr[retstr->strlength] = '\0';

    if ( 0 < rxcInfo.posRecv )
    {
        memmove( rxcInfo.pszRecv, &rxcInfo.pszRecv[ulSize], rxcInfo.posRecv );
    }

    if ( NULLHANDLE != rxcInfo.logFile )
    {
        DosWrite( rxcInfo.logFile, retstr->strptr, retstr->strlength, &ulSize );
    }

    return 0;
}

